#region using
using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.Threading;
using System.Text.RegularExpressions;
#endregion

namespace Cotyounoyume{
	public class Belly
	{
		#region common or UI Properties
		public Atom atom;
		public EBATCore core;
		public bool disableLog;
		#endregion
		
		#region bellyControl
		protected Dictionary<string, Quaternion> initialQuat;
		protected Dictionary<string, Quaternion> targetQuat;
		protected Dictionary<string, float> progress;
		protected Dictionary<string, CycleManager> timing;
		protected Dictionary<string, FreeControllerV3> ctrl;
		protected Dictionary<string, ControllerBackup> backup;
		protected Dictionary<string, bool> isActive;

		protected ControllerBackup lofcLBackup;
		protected ControllerBackup lofcRBackup;
		
		protected string[] keyBreath = {
			"chestControl"
		};
		protected string[] keyOrgasm = {
			"abdomenControl", "abdomen2Control", "pelvisControl", "hipControl"
			, "lThighControl", "lKneeControl", "lFootControl"
			, "rThighControl", "rKneeControl", "rFootControl"			
		};
		
		public Dictionary<string, float> targetStrength = new Dictionary<string, float>
		{
			["headControl"]       = -10
			, ["neckControl"]     = -10
			, ["chestControl"]    = -10
			, ["abdomenControl"]  = -30
			, ["abdomen2Control"] = -30
			, ["pelvisControl"]   = -30
			, ["hipControl"]      = -30
			, ["lThighControl"]   = 30
			, ["lKneeControl"]    = 30
			, ["lFootControl"]    = 30
			, ["rThighControl"]   = 30
			, ["rKneeControl"]    = 30
			, ["rFootControl"]    = 30
		};

		public Dictionary<string, float> closingTimeOffset1 = new Dictionary<string, float>
		{
			["breath"] = 1.2f
		};
		public Dictionary<string, float> closingTimeOffset2 = new Dictionary<string, float>
		{
			["breath"] = 1.2f
		};
		
		protected int waveState;
		protected bool isBellyActive;
		#endregion
		
		private float lastTime;
		
		public Belly(EBATCore core){
			atom = core.atom;
			this.core = core;
		}
		
		public void loadSettings(){
			try{
				targetStrength["chestControl"] = core.settings["General"]["Breath Strength"].AsFloat;
			}catch (Exception e){
				core.loggerE(e);
			}

		}
		
		public void applySettings(){
			setControllerAndProgress();
		}
		
		public void maintenance(){

		}

		#region orgasm functions
		public void setControllerAndProgress(){
			try{
				ctrl   = new Dictionary<string, FreeControllerV3>();
				backup = new Dictionary<string, ControllerBackup>();
				progress = new Dictionary<string, float>();
				timing   = new Dictionary<string, CycleManager>();
				isActive = new Dictionary<string, bool>();

				initialQuat = new Dictionary<string, Quaternion>();
				targetQuat = new Dictionary<string, Quaternion>();
				
				string[] keys = new string[keyBreath.Length + keyOrgasm.Length];
				Array.Copy(keyBreath, 0, keys, 0, keyBreath.Length);
				Array.Copy(keyOrgasm, 0, keys, keyBreath.Length, keyOrgasm.Length);
				
				for(int i = 0; i < keys.Length; i++){
					ctrl[keys[i]] = atom.GetStorableByID(keys[i]) as FreeControllerV3;
					backup.Add(keys[i], new ControllerBackup());
					backup[keys[i]].backUp(ctrl[keys[i]]);
					progress[keys[i]] = 0;
				}
				
				isActive["breath"] = false;
				timing.Add("breath", new CycleManager());					
				
			}catch (Exception e){
				core.loggerE(e);
			}
		}

		public void storePositionAndRotation(){
			try{
				string[] keys = new string[keyBreath.Length + keyOrgasm.Length];
				Array.Copy(keyBreath, 0, keys, 0, keyBreath.Length);
				Array.Copy(keyOrgasm, 0, keys, keyBreath.Length, keyOrgasm.Length);

				initialQuat = new Dictionary<string, Quaternion>();
				
				for(int i = 0; i < keys.Length; i++){
					backup[keys[i]].backUp(ctrl[keys[i]]);
					initialQuat[keys[i]] = ctrl[keys[i]].transform.rotation;
				}
			}catch (Exception e){
				core.loggerE(e);
			}
		}
		
		public void changePositionAndRotationState(){
			try{
				for(int i = 0; i < keyOrgasm.Length; i++){
					ctrl[keyOrgasm[i]].SetRotationStateFromString("Off");
//					ctrl[keyOrgasm[i]].SetPositionStateFromString("Off");
				}
				for(int i = 0; i < keyBreath.Length; i++){
					ctrl[keyBreath[i]].SetRotationStateFromString("On");
					ctrl[keyBreath[i]].SetPositionStateFromString("On");
				}
			}catch (Exception e){
				core.loggerE(e);
			}
		}		
		
		public void restoreStatus(){
			try{
				for(int i = 0; i < keyOrgasm.Length; i++){
					ctrl[keyOrgasm[i]] = backup[keyOrgasm[i]].recoveryStatus(ctrl[keyOrgasm[i]]);
				}
				for(int i = 0; i < keyBreath.Length; i++){
					ctrl[keyBreath[i]] = backup[keyBreath[i]].recoveryStatus(ctrl[keyBreath[i]]);
				}
			}catch (Exception e){
				core.loggerE(e);
			}
		}
		
		public void restorePosition(){
			try{
				for(int i = 0; i < keyOrgasm.Length; i++){
					ctrl[keyOrgasm[i]] = backup[keyOrgasm[i]].recoveryPosition(ctrl[keyOrgasm[i]]);
				}
				for(int i = 0; i < keyBreath.Length; i++){
					ctrl[keyBreath[i]] = backup[keyBreath[i]].recoveryPosition(ctrl[keyBreath[i]]);
				}
			}catch (Exception e){
				core.loggerE(e);
			}
		}
		
		public void restoreRotation(){
			try{
				for(int i = 0; i < keyOrgasm.Length; i++){
					ctrl[keyOrgasm[i]] = backup[keyOrgasm[i]].recoveryRotation(ctrl[keyOrgasm[i]]);
				}
				for(int i = 0; i < keyBreath.Length; i++){
					ctrl[keyBreath[i]] = backup[keyBreath[i]].recoveryRotation(ctrl[keyBreath[i]]);
				}
			}catch (Exception e){
				core.loggerE(e);
			}
		}
		
		public void restorePositionAndRotation(){
			try{
				for(int i = 0; i < keyOrgasm.Length; i++){
					ctrl[keyOrgasm[i]] = backup[keyOrgasm[i]].recovery(ctrl[keyOrgasm[i]]);
				}
				for(int i = 0; i < keyBreath.Length; i++){
					ctrl[keyBreath[i]] = backup[keyBreath[i]].recovery(ctrl[keyBreath[i]]);
				}
			}catch (Exception e){
				core.loggerE(e);
			}
		}
		
		public void updatePosition(float time, string[] keys, string position){
			try{
				if(isActive[position]){
					return;
				}else{
					if(core.getStage() >= 2){
						if(core.isEmbarassed){
							core.expression2.update();
						}
						core.sound.playSound("breath", Time.time);
					}

					core.guess.CheckTrunk();
//					core.debug("updatePosition:" + position);
					isActive[position] = true;
					float time1 = time + closingTimeOffset1[position] * UnityEngine.Random.Range(0.7f,1.0f) * AdjustInterval();
					float time2 = time1 + closingTimeOffset2[position] * UnityEngine.Random.Range(0.7f,1.0f) * AdjustInterval();
					timing[position].setTimer(time, time1, time2);

					float tmpInitialStrength;
					float tmpTargetStrength;
					FreeControllerV3 baseController = getBaseController(position);
					for(int i = 0; i < keys.Length; i++){
//						tmpInitialStrength = ctrl[keys[i]].transform.localEulerAngles.x;
//						tmpTargetStrength  = tmpInitialStrength + targetStrength[keys[i]] * UnityEngine.Random.Range(0.8f,1.0f) * AdjustStrength();
						tmpTargetStrength  = targetStrength[keys[i]] * UnityEngine.Random.Range(0.8f,1.0f) * AdjustStrength();
//						targetQuat[keys[i]] = Quaternion.AngleAxis(tmpTargetStrength, baseController.transform.right) * baseController.transform.rotation;
						targetQuat[keys[i]] = Quaternion.AngleAxis(tmpTargetStrength, core.trunk.transform.right) * core.trunk.transform.rotation;
					}
				}
			}catch (Exception e){
				core.loggerE(e);
			}
		}

		public FreeControllerV3 getBaseController(string position){
			FreeControllerV3 result;
			if(position == "breath"){
				result = ctrl["chestControl"];
			}else{
				result = null;
			}
			return result;
		}
		
		public void calcProgress(float time, string position){
			try{
				progress[position] = timing[position].getProgress(time);
				
				if(timing[position].getState(time) == 0){
					isActive[position] = false;
				}
			}catch (Exception e){
				core.loggerE(e);
			}
		}
		
		public void changeJoint(float time, string[] keys, string position){
			try{
				if(!timing.ContainsKey(position)){
					core.debug("key:" + position + " not found.");
				}
				int state = timing[position].getState(time);
				for(int i = 0; i < keys.Length; i++){
					if(state != 0){
						if(!ctrl.ContainsKey(keys[i])){
							core.debug("ctrl key:" + keys[i] + " not found.");
						}
						if(!initialQuat.ContainsKey(keys[i])){
							core.debug("initialQuat key:" + keys[i] + " not found.");
						}
						if(!targetQuat.ContainsKey(keys[i])){
							core.debug("targetQuat key:" + keys[i] + " not found.");
						}
						if(!progress.ContainsKey(position)){
							core.debug("progress key:" + position + " not found.");
						}
						ctrl[keys[i]].transform.rotation = Quaternion.Slerp(initialQuat[keys[i]], targetQuat[keys[i]], progress[position]);
					}
				}
			}catch (Exception e){
				core.loggerE(e);
			}
		}

		public float AdjustInterval(){
			float stageCoefficient = (float)core.getStage() * 0.1f;
			return (1 / (1 + stageCoefficient));
		}
		
		public float AdjustStrength(){
			float stageCoefficient = (float)core.getStage() * 0.2f;
			return 1 + stageCoefficient;
		}
		
		public void manage(float time){
			try{
				if(time == lastTime){
					time += 0.001f;
				}else if(time < lastTime){
					time = lastTime + 0.001f;
				}
				lastTime = time;

				if(core.getStage() < 1){
					isActive["breath"] = false;
					return;
				}
				if(isActive["breath"] == false){
					updatePosition(time, keyBreath, "breath");
					core.eyesControl.update(time);
					return;
				}

				calcProgress(time, "breath");
				changeJoint(time, keyBreath, "breath");
			}catch (Exception e){
				core.loggerE(e);
			}
		}
		
		public void bellyBump(float strength, float time){
			try{
				for(int i = 0; i < keyOrgasm.Length; i++){
					if(keyOrgasm[i].StartsWith("l")){
						ctrl[keyOrgasm[i]].jointRotationDriveZTarget = targetStrength[keyOrgasm[i]] * strength * -1;
					}else if(keyOrgasm[i].StartsWith("r")){
						ctrl[keyOrgasm[i]].jointRotationDriveZTarget = targetStrength[keyOrgasm[i]] * strength;						
					}else{
						ctrl[keyOrgasm[i]].jointRotationDriveXTarget = targetStrength[keyOrgasm[i]] * strength;
					}
				}
			}catch (Exception e){
				core.loggerE(e);
			}
		}
		
		public void bellyDents(float strength, float time){
			try{
				for(int i = 0; i < keyOrgasm.Length; i++){
					if(keyOrgasm[i].StartsWith("l")){
						ctrl[keyOrgasm[i]].jointRotationDriveZTarget = 0f;
					}else if(keyOrgasm[i].StartsWith("r")){
						ctrl[keyOrgasm[i]].jointRotationDriveZTarget = 0f;
					}else{
						ctrl[keyOrgasm[i]].jointRotationDriveXTarget = 0f;
					}
				}			
			}catch (Exception e){
				core.loggerE(e);
			}
		}
		
		public void OnDestroy(){
			RemoveBackup();
			RemoveTiming();
		}
		
		public void RemoveBackup(){
			foreach(var kvp in backup){
				if(backup[kvp.Key] != null){
					backup[kvp.Key] = null;
				}
			}
			backup = null;
		}
		
		public void RemoveTiming(){
			foreach(var kvp in timing){
				if(timing[kvp.Key] != null){
					timing[kvp.Key] = null;
				}
			}
			timing = null;			
		}
		#endregion
	}
}