#region using
using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.Threading;
using System.Text.RegularExpressions;
#endregion

namespace Cotyounoyume{
	public class ClothesManager
	{
		public ClothesStorage[] clothesStorages;
		public Atom atom;
		public DAZCharacterSelector geometryForClothes;
		
		public ClothesManager(){
			clothesStorages = new ClothesStorage[6];
			for(int i = 0; i < 6; i++){
				clothesStorages[i] = new ClothesStorage();
			}
		}
		
		public void initialize(Atom atom){
			this.atom = atom;
			JSONStorable geometryStorable = this.atom.GetStorableByID("geometry");
			geometryForClothes = geometryStorable as DAZCharacterSelector;
		}
		
		public void equipClothes(int stage){
			try{
//				debug("equipClothes: stage=" + stage);
				if(stage < 0 || stage > 5){
					return;
				}
				for(int i = 0; i < clothesStorages[stage].clothes.Count; i++){
//					debug(clothesStorages[stage].clothes[i].uid);
					geometryForClothes.SetActiveClothingItem(clothesStorages[stage].clothes[i], active: true);
				}
			}catch(Exception e){
				loggerE(e);
			}
		}
		
		public void unequipClothes(int stage){
			try{
//				debug("unequipClothes: stage=" + stage);
				if(stage < 0 || stage > 5){
					return;
				}
				for(int i = 0; i < clothesStorages[stage].clothes.Count; i++){
//					debug(clothesStorages[stage].clothes[i].uid);
					geometryForClothes.SetActiveClothingItem(clothesStorages[stage].clothes[i], active: false);
				}				
			}catch(Exception e){
				loggerE(e);
			}
		}
		
		public List<DAZClothingItem> getClothes(){
			List<DAZClothingItem> clothes = geometryForClothes.clothingItems.ToList();
			List<DAZClothingItem> myClothes = new List<DAZClothingItem>();
			foreach (DAZClothingItem dAZClothingItem in clothes){
				if(dAZClothingItem.active){
					myClothes.Add(dAZClothingItem);
				}
			}
			
			return myClothes;
		}
		
		public List<string> getClothUids(int stage){
			List<string> myClothes = clothes2uids(clothesStorages[stage].clothes);			
			return myClothes;
		}
		
		public void setClothes(int stage){
			try{
				List<DAZClothingItem> tmpClohtes = getClothes();
				clothesStorages[stage].setClothes(tmpClohtes);				
			}catch(Exception e){
				loggerE(e);
			}
		}
		
		public void showClothes(int stage){
			try{
//				debug("stage:" + stage);
				clothesStorages[stage].showClothes();
			}catch(Exception e){
				loggerE(e);
			}
		}
		
		public void setClothesByUids(int stage, List<string> uids){
			try{
				List<DAZClothingItem> tmpClohtes = this.uids2clothes(uids);
				clothesStorages[stage].setClothes(tmpClohtes);
			}catch(Exception e){
				loggerE(e);
			}
		}
		
		public void clearClothes(int stage){
			clothesStorages[stage].clearClothes();
		}
		
		public void undressAll(){
			geometryForClothes.RemoveAllClothing();
		}
		
		public List<string> clothes2uids(List<DAZClothingItem> clothes){
			int num = clothes.Count;
			List<string> uids = new List<string>();
			for(int i = 0; i < num; i++){
				uids.Add(clothes[i].uid);
			}
			return uids;
		}
		
		public List<DAZClothingItem> uids2clothes(List<string> uids){
			Dictionary<string, bool> uidsList;
			uidsList = new Dictionary<string, bool>();			
			for(int i = 0; i < uids.Count; i++){
				uidsList[uids[i]] = true;
			}
			
			List<DAZClothingItem> clothes = new List<DAZClothingItem>();

			List<DAZClothingItem> clothesAll = geometryForClothes.clothingItems.ToList();
			for(int i = 0; i < clothesAll.Count; i++){
				if(uidsList.ContainsKey(clothesAll[i].uid)){
					clothes.Add(clothesAll[i]);
				}
			}
			
			return clothes;
		}
		
		public void debug(string str){
			SuperController.LogMessage("debug: " + str);
		}
		
		public void loggerE(Exception e){
			SuperController.LogMessage("*** Exception:" + e);
		}
		
		public void OnDestroy(){
			RemoveClothesStorage();
		}
		
		public void RemoveClothesStorage(){
			for(int i = 0; i < 6; i++){
				clothesStorages[i] = null;
			}
			clothesStorages = null;			
		}
	}
	
	public class ClothesStorage
	{
		public List<DAZClothingItem> clothes;
		
		public ClothesStorage(){
			clothes = new List<DAZClothingItem>();
		}
		
		public void setCurrentClothes(Atom atom){
			JSONStorable geometryStorable = atom.GetStorableByID("geometry");
			DAZCharacterSelector geometryForClothes = geometryStorable as DAZCharacterSelector;
			List<DAZClothingItem> clothes = geometryForClothes.clothingItems.ToList();
			for (int i = 0; i < clothes.Count; i++){
				DAZClothingItem c = clothes[i];
				if(c.active){
					
				}
			}
		}
		
		public void setClothes(List<DAZClothingItem> clothes){
			this.clothes = new List<DAZClothingItem>();
			for(int i = 0; i < clothes.Count; i++){
				this.clothes.Add(clothes[i]);
			}
		}
		
		public void showClothes(){
			if(this.clothes.Count == 0){
				SuperController.LogMessage("  clothes not found.");				
			}
			for(int i = 0; i < this.clothes.Count; i++){
				SuperController.LogMessage("  Name: " + this.clothes[i].displayName);
//				SuperController.LogMessage("  Uid : " + this.clothes[i].uid);
			}
		}
		
		public void clearClothes(){
			this.clothes = new List<DAZClothingItem>();
		}
		
		public void debug(string str){
			SuperController.LogMessage("debug: " + str);
		}
		
		public void loggerE(Exception e){
			SuperController.LogMessage("*** Exception:" + e);
		}
	}
}