using System;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.XR;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using Object = UnityEngine.Object;
using System.Text.RegularExpressions;

namespace Cotyounoyume
{
    public class CustomUnityAssetItem : MonoBehaviour 
    {
		public Atom atom;
		public EBATCore core;

		public CustomUnityAssetItem(EBATCore core){
			atom = core.atom;
			this.core = core;
		}
		
		public string GetAtomId(string cuaName){
			return core.GenerateAtomName(cuaName, core.atom.uid);			
		}

		public void PrintAllAtomId(){
			foreach(KeyValuePair<string, Atom> kvp in core.generatedAtomsDictionary){
				core.debug("generatedAtomsDictionary: Key: " + kvp.Key + ", Value: " + kvp.Value);
			}
		}

		public Atom GetCUAAtom(string cuaName){
			string atomId = GetAtomId(cuaName);
			if(core.generatedAtomsDictionary.ContainsKey(atomId)){
				return core.generatedAtomsDictionary[atomId];
			}
			return null;
		}
		
		public void RemoveCUA(string cuaName){
			Atom cuaAtom = GetCUAAtom(cuaName);
			string atomId = GetAtomId(cuaName);
			if(cuaAtom != null){
				SuperController.singleton.RemoveAtom(cuaAtom);
				core.generatedAtomsDictionary.Remove(atomId);
			}else{
				core.logger("Can't RemoveCUA: cuaName: " + cuaName + " is null");
			}
		}
		
		public void ResetPositionCUA(string cuaName, string bodyName, float upOffset, float forwardOffset, float rightOffset){
			try{
				Atom tmpatom = GetCUAAtom(cuaName);
				if(tmpatom == null){
					return;
				}
				Vector3 pos = Utility.getPositionFromTargetBodynameWithOffset(core.atom, bodyName, upOffset, forwardOffset, rightOffset, 0f);
				Quaternion rot = Utility.getRotationFromTargetBodyname(core.atom, bodyName);
				tmpatom.transform.SetPositionAndRotation(pos, rot);
			}catch(Exception e){
				core.loggerE(e);
			}
		}
		
		public void summonAtomOnly(string name, string bodyName, bool debugOn = true){
			if(GetCUAAtom(name) != null){
				RemoveCUA(name);				
			}
			
			core.StartCoroutine(core.CreateAtom("CustomUnityAsset", name, (tmpatom)=>
			{
				ResetPositionCUA(name, bodyName, 0f, 0f, 0f);
				
				if(debugOn == false){
					tmpatom.hidden = true;
					tmpatom.mainController.canGrabPosition = false;
					tmpatom.mainController.canGrabRotation = false;
				}
			}));			
		}

		public void summonCUA(string cuaName, string url, string assetName, string bodyName, float upOffset, float forwardOffset, float rightOffset, bool femaleOnly = true, bool debugOn = true, bool bSetParent = true){
			if(core.gender=="Male" && femaleOnly == true){
				return;
			}

			if(GetCUAAtom(cuaName) != null){
				RemoveCUA(cuaName);				
			}

			core.StartCoroutine(core.CreateAtom("CustomUnityAsset", cuaName, (tmpatom)=>
			{
				if(bSetParent){
					Utility.setParent(tmpatom, atom, bodyName);
				}
				
				string fullpath = core.myPath.Replace("Custom/Scripts/cotyounoyume/ExpressionBlushingAndTears", "") + url;
				if(url != ""){
					Utility.setAssetBundleUrl(tmpatom, fullpath);
				}
				if(assetName != ""){
					Utility.resetAssetBundleName(tmpatom, assetName);
				}
				
				ResetPositionCUA(cuaName, bodyName, upOffset, forwardOffset, rightOffset);
				
				if(debugOn == false){
					tmpatom.hidden = true;
					tmpatom.mainController.canGrabPosition = false;
					tmpatom.mainController.canGrabRotation = false;
				}
			}));			
		}
	}
}